package domain;

import java.util.StringTokenizer;

public class Time extends Clock
implements Comparable<Time> {
	public Time() {
		super(12, 0);
	}
	public Time(int hour, int minute) throws BadHourException, BadMinuteException {
		if (hour < 0 || hour >= HOURS_PER_DAY) throw new BadHourException(hour);
		h = hour;
		if (minute < 0 || minute >= MINUTES_PER_HOUR) throw new BadMinuteException(minute);
		m = minute;
	}
	public Time(String s) throws BadHourException, BadMinuteException, BadFormatException {
		if (!s.matches("^[0-2]{0,1}[0-9]:[0-5][0-9]$")) throw new BadFormatException(s);
		StringTokenizer st = new StringTokenizer(s, ":");
		h = Integer.parseInt(st.nextToken());
		if (h < 0 || h >= HOURS_PER_DAY) throw new BadHourException(h);
		m = Integer.parseInt(st.nextToken());
		if (m < 0 || m >= MINUTES_PER_HOUR) throw new BadMinuteException(m);
	}
	public void oneHourMore() {
		h = (h + 1) % HOURS_PER_DAY;
	}
	public void oneHourLess() {
		h--;
		if (h < 0) h = HOURS_PER_DAY - 1;
	}
	public void oneMinuteLess() {
		m--;
		if (m < 0 ) {
			m = MINUTES_PER_HOUR - 1;
			oneHourLess();
		}
	}
	public String toString() {
		String s = ((h < 10) ? "0" : "");
		s += h;
		s += ":";
		s += (m < 10) ? "0" : "";
		s += m;
		return s;
	}
	public int compareTo(Time o) {
		if (h == o.h)
			return m - o.m;
		if (h < o.h)
			return 60 * (h - o.h - 1) + (MINUTES_PER_HOUR - m) + o.m;
		return 60 * (h - o.h - 1) + (MINUTES_PER_HOUR - o.m) + m;
	}
}
