package util;

import java.util.Iterator;

public class List<ElementType extends Comparable<ElementType>>
implements ListSpecification<ElementType>  {
	private class ListElement {
		public ElementType content;
		public ListElement next;
		public ListElement previous;
		public ListElement(ElementType element) {
			content = element;
			next = previous = null;
		}
	}
	private class IteratorFromBeginning implements Iterator<ElementType> {
		private ListElement current;
		public IteratorFromBeginning() {
			current = first;
		}
		public boolean hasNext() {
			return current != null;
		}
		public ElementType next() {
			ElementType value = current.content;
			current = current.next;
			return value;
		}
	}
	private class IteratorFromEnd implements Iterator<ElementType> {
		private ListElement current;
		public IteratorFromEnd() {
			current = last;
		}
		public boolean hasNext() {
			return current != null;
		}
		public ElementType next() {
			ElementType value = current.content;
			current = current.previous;
			return value;
		}
	}
	private class IteratorWhere implements Iterator<ElementType> {
		private ListElement current;
		private Test<ElementType> test;
		public IteratorWhere(Test<ElementType> t) {
			current = first;
			test = t;
			avancerAuProchainQui();
		}
		private void avancerAuProchainQui() {
			while (current != null && !test.successfullFor(current.content))
				current = current.next;
		}
		public boolean hasNext() {
			return current != null;
		}
		public ElementType next() {
			ElementType value = current.content;
			current = current.next;
			avancerAuProchainQui();
			return value;
		}
	}
	private class BubbleSort implements Strategy {
		public void execute() {
			if (numberOfElements < 2) return;
			boolean exchange = true;
			ListElement courant, suivant;
			while(exchange) {
				courant = first;
				suivant = courant.next;
				exchange = false;
				while (suivant != null) {
					if (courant.content.compareTo(suivant.content) > 0) {
						exchangeContentsOf(courant, suivant);
						exchange = true;
					}
					courant = suivant;
					suivant = courant.next;
				}
			}
		}
	}
	private class SelectionSort implements Strategy {
		public void execute() {
			int passe;
			ListElement debut;
			for(passe = 1, debut = first; passe < numberOfElements; passe++, debut = debut.next) {
				ListElement min = debut;
				ListElement courant = debut.next;
				while (courant != null) {
					if (min.content.compareTo(courant.content) > 0)
						min = courant;
					courant = courant.next;
				}
				exchangeContentsOf(debut, min);
			}
		}
	}
	private ListElement first, last;
	private boolean fromBeginning = true;
	private int numberOfElements;
	public List() {
		first = last = null;
		numberOfElements = 0;
	}
	public void add(ElementType element) {
		ListElement newListElement = new ListElement(element);
		if (first == null) {
			first = newListElement;
		} else {
			last.next = newListElement;
			newListElement.previous = last;
		}
		last = newListElement;
		numberOfElements++;
	}
	private void remove(ListElement current) {
		if (current == first && current == last) {
			first = last = null;
			numberOfElements = 0;
		} else {
			if (current != first && current != last) {
				current.previous.next = current.next;
				current.next.previous = current.previous;
			}
			if (current == first) {
				first = current.next;
				first.previous = null;
			}
			if (current == last) {
				last = current.previous;
				last.next = null;
			}
			numberOfElements--;
		}
		current.next = current.previous = null;
	}
	public void remove(ElementType element) {
		ListElement current = first;
		while (current != null && current.content != element) {
			current = current.next;
		}
		if (current != null) remove(current);
	}
	public void remove(int index) {
		ListElement current;
		if (index < size() / 2) {
			current = first;
			while (current != null && index > 0) {
				current = current.next;
				index--;
			}
		} else {
			index = size() - index - 1;
			current = last;
			while (current != null && index > 0) {
				current = current.previous;
				index--;
			}
		}
		remove(current);
	}
	public int size() {
		return numberOfElements;
	}
	private ElementType getFromBeginning(int index) {
		ListElement current = first;
		while (current != null && index > 0) {
			System.err.print(" " + current.content + " ");
			current = current.next;
			index--;
		}
		System.err.println(" " + current.content);
		return current.content;
	}
	private ElementType getFromEnd(int index) {
		ListElement current = last;
		while (current != null && index > 0) {
			System.err.print(" " + current.content + " ");
			current = current.previous;
			index--;
		}
		return current.content;
	}
	public ElementType get(int index) {
		if (index < size() / 2)
			return getFromBeginning(index);
		return getFromEnd(size() - index - 1);
	}
	public boolean isEmpty() {
		return first == null;
	}
	public boolean contents(ElementType element) {
		ListElement current = first;
		while (current != null && current.content != element) {
			current = current.next;
		}
		return (current != null);
	}
	public void iterateFromBeginning(boolean value) {
		fromBeginning = value;
	}
	public Iterator<ElementType> iterator() {
		if (fromBeginning) return new IteratorFromBeginning();
		else return new IteratorFromEnd();
	}
	public Iterator<ElementType> iteratorWhere(Test<ElementType> t) {
		return new IteratorWhere(t);
	}
	private void exchangeContentsOf(ListElement le1, ListElement le2) {
		ElementType svg = le1.content;
		le1.content = le2.content;
		le2.content = svg;
	}
	public void sort() {
		if (Math.random() > 0.5d) { //choix alatoire de la stratgie
			System.out.println("tri bulle...");
			new BubbleSort().execute();
		} else {
			System.out.println("tri slection...");
			new SelectionSort().execute();
		}
	}
}
